﻿using System;
using System.Collections.Generic;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;
using CommonMP.HYSSOP.Interface.HSData;

namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>OHyMoS2次元データファイル入力データクラス</para>
	/// </summary>
	[Serializable]
	public class FileInputMatrixInfo : McCalInfo
	{
		/// <summary> 入力ファイル名称 </summary>
		public String m_csReadFileNameWithPath;

		/// <summary> データ一時保管 </summary>
		public LinkedList<Input2DRecord> m_csDataList;

		/// <summary> 計算時刻より先まで送信する時間 </summary>
		public double m_dMarginTime;

		/// <summary> 入力時の基準年月日（パラメータより取得）</summary>
		public HySTime m_csOriginTime;

		/// <summary> 入力基準　年・月・日 </summary>
		public long m_lOriginYear, m_lOriginMonth, m_lOriginDay;

		/// <summary> 最後に送信した時刻</summary>
		public HySTime m_csLastOutTime;

		/// <summary> 入力ファイルのタイムスタンプを保存する</summary>
		public HySTime m_csFileTime;

		/// <summary><para>method outline:</para>
		/// <para>自己複製</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> Clone( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>同一内容で　別インスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>収束演算時、特定の時刻に状態を戻す場合等に使用する</para>
		/// </remarks>
		public override McCalInfo Clone()
		{
			FileInputMatrixInfo csRtn = new FileInputMatrixInfo();
			csRtn.CopyInfo(this);
			return csRtn;
		}
		/// <summary><para>method outline:</para>
		/// <para>引数で与えられた情報を自分にコピーを行う</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> bool bRtn = CopyInfo(csOrgInfo) </para>
		/// </example>
		/// <param name="csOrgInfo">コピー元情報</param>
		/// <returns> bool true :正常 , false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool CopyInfo(McCalInfo csOrgInfo)
		{
			FileInputMatrixInfo csOrgDt = (FileInputMatrixInfo)csOrgInfo;
			this.m_csReadFileNameWithPath = (String)(csOrgDt.m_csReadFileNameWithPath.Clone());
			
			this.m_csDataList = csOrgDt.m_csDataList;
			this.m_dMarginTime = csOrgDt.m_dMarginTime;
			this.m_csOriginTime = csOrgDt.m_csOriginTime.Clone();
			this.m_lOriginYear = csOrgDt.m_lOriginYear;
			this.m_lOriginMonth = csOrgDt.m_lOriginMonth;
			this.m_lOriginDay = csOrgDt.m_lOriginDay;
			this.m_csLastOutTime = csOrgDt.m_csLastOutTime.Clone();
			this.m_csFileTime = csOrgDt.m_csFileTime.Clone();
			return true;
		}

	}

	/// <summary>
	/// データ格納用クラス
	/// </summary>
	public class Input2DRecord : HySDataRoot
	{
		/// <summary> 最終出力時刻 </summary>
		public HySTime m_csTime;
		/// <summary> データ一時保管配列 </summary>
		public double[,] m_dDT;
		/// <summary>2次元データの横の大きさ</summary>
		public long dx;
		/// <summary>2次元データの縦の大きさ</summary>
		public long dy;

		/// <summary><para>method outline:</para>
		/// <para>指定された大きさの2次元データ配列を生成するコンストラクター</para>
		/// </summary>
		/// <returns>生成されたインスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <param name="ax">X</param>
		/// <param name="ay">Y</param>
		/// </remarks>
		public Input2DRecord(int ax, int ay)
		{
			m_dDT = new double[ax, ay];
			dx = ax;
			dy = ay;
		}
	}
}
