﻿using System;
using System.Collections.Generic;
using CommonMP.HYMCO.CoreImpl.Data;
using CommonMP.HYMCO.CoreImpl.Model;
using CommonMP.HYMCO.CoreImpl.Tool;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>OHyMoSデータファイル出力モデルクラス</para>
	/// </summary>
	public class FileOutputBasic : McBasicFileOutCtl
	{
		/// <summary> 内部変数 </summary>
		FileOutputBasicInfo mInf = new FileOutputBasicInfo();


		/// <summary><para>method outline:</para>
		/// <para>入力側の接続情報チェック</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = ReceiveConnectionCheck(ref csErrorInf)</para>
		/// </example>
		/// <param name="csErrorInf">エラー出力</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>=true:正常、=false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>受信するデータが自モデルが期待している情報か否かをチェックする</para>
		/// </remarks>
		protected override bool ReceiveConnectionCheck(ref McStructErrorInfo csErrorInf, long lInputDataNum, McReceiveCellDataIF[] csInputCellData)
		{
			bool bRtn = true;
			
			// 入力端子数のチェック
			if (lInputDataNum == 0)
			{
				csErrorInf.AddCheckErrorData(this.m_csElement.GetID(), ToolDefine.FILE_OUT_KIND, "No Receive Port");
				bRtn = false;
			}
			else if (lInputDataNum > 1)
			{
				csErrorInf.AddCheckErrorData(this.m_csElement.GetID(), ToolDefine.FILE_OUT_KIND, "Too many Receive Ports. (Not support multi-Receive Ports)");
				bRtn = false;
			}
			else if (lInputDataNum == 1)
			{
				if (csInputCellData[0].GetReceivePatternID().Equals(ToolDefine.IN_OUT_PATTERN_ANY) == false)
				{
					// 予期しない接続が行われている場合
					csErrorInf.AddCheckErrorData(this.GetID(), ToolDefine.FILE_OUT_KIND,
						"Unexpected receive data type. (Received from " + csInputCellData[0].GetUpperElementID().ToString() + ")");
					bRtn = false;
				}
			}
			return bRtn;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>Initialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool Initialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize","<<Start>>");

            // m-suto 2015/12/25 start
            // 取得すべきセル内の位置情報を取得
            mInf.m_lOutDataIdxInCell = csInputCellData[0].GetCellDataGetter().GetIdx()[0];
            // m-suto 2015/12/25 end    
            
			// 情報の初期化
			mInf.m_csLastINTime = new HySTime(0);
			mInf.m_csLastOutTime = new HySTime(0);

			// 出力基準時間 m_csOriginTimeの生成
			mInf.m_csOriginTime = HySCalendar.CreateTime(mInf.m_lOriginYear, mInf.m_lOriginMonth, mInf.m_lOriginDay, 0, 0, 0);

			// 計算開始時間の記憶
			mInf.m_csStartTime = new HySTime(this.m_csSimTime.TotalSec());

			mInf.m_csDataList = new LinkedList<OutputBasicRecord>();

			// 基準時間が計算開始時間より先になっている場合
			// 出力される時間が負になってしまうので警告を出しておく
			if (mInf.m_csOriginTime.After(mInf.m_csStartTime))
			{
				McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "Caution:DataOriginTime is earlier than Simulation StartTime");	
			}

			string sFileName = mInf.m_csOutFileNameWithPath + "";
			
			// ファイル名の中に時間変換記号が見つかった場合、現在の時間（分まで）が代入される
			if (sFileName.IndexOf("&t") >= 0)
			{
				DateTime nowtime = System.DateTime.Now;
				string myformat = "yyMMdd-HHmmss";
				String stime = nowtime.ToString(myformat);
				sFileName = sFileName.Replace("&t", stime);

			}

			HySCellData csCell = csInputCellData[0].GetInterpolatedCell(0);
			long lCellDataNum = csCell.m_dData.Length;

			// ファイル名の中にワイルドカード要素が見つかった場合、セル一括出力モードになる
			if (sFileName.IndexOf('*') >= 0)
			{	
				if (lCellDataNum >= 1)
				{
					mInf.m_lAllCellNum = lCellDataNum;
					mInf.m_csAllOutFile = new HySFile[mInf.m_lAllCellNum];
					mInf.m_csAllOutFileName = new String[mInf.m_lAllCellNum];
					
					for (int icell = 0; icell < lCellDataNum; icell++)
					{
						mInf.m_csAllOutFileName[icell] = sFileName.Replace("*", icell.ToString());
					
						// HySCellDataGetter csCell = csInputCellData[0].GetCellDataGetter();
						// HySDataCharacteristicInCell test = csInputCellData[0].GetReceiveTranInfoPattern().GetCellDataCharacteristic();
					}
				}
				else
				{
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "Illegal Cell Data.");
					return false;
				}
			}

			else
			{
                // m-suto 2015/12/25 start
                // mInf.m_lOutDataIdx ⇒ mInf.m_lOutDataIdxInCell に変更
                if (mInf.m_lOutDataIdxInCell >= lCellDataNum)
                // m-suto 2015/12/25 end
                {
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "Illegal Property 'Output Celldata Index'.");
					return false;
				}
				mInf.m_lAllCellNum = 1;
				mInf.m_csAllOutFileName = new String[mInf.m_lAllCellNum];
				mInf.m_csAllOutFile = new HySFile[mInf.m_lAllCellNum];

				// 出力ファイル設定
				mInf.m_csAllOutFileName[0] = mInf.m_csOutFileNameWithPath;
				
			}

			int i = 0;
			try
			{
				for (i = 0; i < mInf.m_lAllCellNum; i++)
				{
					mInf.m_csAllOutFile[i] = new HySFile(mInf.m_csAllOutFileName[i]);

					// 出力ファイル初期生成
					mInf.m_csAllOutFile[i].Open(HySFile.OPEN_MODE.CREATE_NEW, HySFile.READ_WRITE_MODE.WRITE, HySFile.DIRECTORY_MODE.MK_DIR);
					// ヘッダーを作成
					mInf.m_csAllOutFile[i].WriteText("# OutputBasicFile,Ver1.0");  // ヘッダー

					mInf.m_csAllOutFile[i].WriteText("# ElementID:" + this.GetID().ToString());  // ヘッダー
					string sTime = HySCalendar.GetString(mInf.m_csOriginTime, HySCalendar.FORMAT.lSW_YEAR).ToString();
					mInf.m_csAllOutFile[i].WriteText("# 基準時間 " + sTime);  // ヘッダー

					mInf.m_csAllOutFile[i].WriteText("# Time, Data");
					mInf.m_csAllOutFile[i].WriteText("# ");
					mInf.m_csAllOutFile[i].WriteText("# ");

					mInf.m_csAllOutFile[i].WriteText("# ");
					mInf.m_csAllOutFile[i].Close();


				}
			}
			catch(Exception e)
			{
				McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "IO Error FileName:" + mInf.m_csAllOutFileName[i] + "," + e.Message);
				return false;
			}
			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>OnlineInitialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>bool</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool OnlineInitialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデル演算結果を外部のエレメントに対して公開する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>long lRtn = DataFusion( lOutputDataNum, ref csOutputCellData)</para>
		/// </example>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override long DataFusion(long lOutputDataNum, ref McSendCellDataIF[] csOutputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "DataFusion", "<<Start>>");
			// データ取得
			if (this.ReadData() < 0)
			{
				McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "DataFusion", "Error in ReadData Section.");
				return -1;
			}

            // m-suto 2015/12/25 start
            // CommonMP ver1.4で正しく出力できない問題があったため、
            // WriteData()前に実施していた1000レコードのバッファリングを削除
            // m-suto 2015/12/25 end
            if (this.WriteData() < 0)
	        {
		        McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "DataFusion", "Error in WriteData Section.");
		        return -1;
	        }

			return 0;
		}
		 /// <summary><para>method outline:</para>
		/// <para>演算終了時に呼ばれる</para>
		/// </summary>
		/// <returns>=true:正常 false:異常</returns>
		/// <exception cref="">無し</exception>
		public override bool CompleteCalculation()
		{
			// データが残っている場合は残りすべてを書き出す処理
			if (mInf.m_csDataList.Count > 0)
			{
				if (this.WriteData() < 0)
				{
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "CompleteCalculation", "Error in WriteData Section.");
					return false;
				}
			}
			return true;
		}

		/// <summary><para>method outline:</para>
		/// <para>保存しているデータをファイルへと書きこむ処理</para>
		/// </summary>
		/// <returns>=true:正常 false:異常</returns>
		/// <exception cref="">無し</exception>
		protected long WriteData()
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "WriteData", "<<Start>>");
			long lDtNum = mInf.m_csDataList.Count;  // 保管中の情報の数
			if (lDtNum < 1)
			{
				// 保管情報が一つもない場合は終了
				return 0;
			}

			OutputBasicRecord csData = null;

			// 出力ファイルオープン
			for (int i = 0; i < mInf.m_lAllCellNum; i++)
			{
				if (mInf.m_csAllOutFile[i].Open(HySFile.OPEN_MODE.APPEND, HySFile.READ_WRITE_MODE.WRITE, HySFile.DIRECTORY_MODE.NOT_MK_DIR) != 0)
				{
					McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "DataFusion", "Cannot Open File:" + mInf.m_csAllOutFileName[i]);
					return -1;
				}
			}

			// 保管中の情報分繰り返す
			for (int iLp = 0; iLp < lDtNum; iLp++)
			{
				// 保管情報を1レコードづつ取り出す
				csData = mInf.m_csDataList.First.Value;

				if (csData.m_csTime.TotalSec() >= mInf.m_csLastOutTime.TotalSec() + mInf.m_csOutTimeRange.TotalSec())
				{
					string sStr = "";
					for (int i = 0; i < mInf.m_lAllCellNum; i++)
					{
						// ファイルへ書き込み
						sStr = (csData.m_csTime.TotalSec() - mInf.m_csOriginTime.TotalSec()).ToString();
						if (mInf.m_lAllCellNum == 1)
						{
                            // m-suto 2015/12/25 start
                            // mInf.m_lOutDataIdx ⇒ mInf.m_lOutDataIdxInCell に変更
                            sStr += " " + (csData.m_dDT[mInf.m_lOutDataIdxInCell]).ToString();
                            // m-suto 2015/12/25 end
						}
						else
						{
							sStr += " " + (csData.m_dDT[i]).ToString();
						}
						mInf.m_csAllOutFile[i].WriteText(sStr);
					}
					mInf.m_csLastOutTime.SetTime(csData.m_csTime);
				}

				// 読み出した情報は一時保管情報から削除する
				mInf.m_csDataList.RemoveFirst();
			}

			for (int i = 0; i < mInf.m_lAllCellNum; i++)
			{
				// 出力ファイルを閉じる
				mInf.m_csAllOutFile[i].Close();
			}
			return 0;
		}

		/// <summary><para>method outline:</para>
		/// <para>入力された伝送データの取得し一時保管しておく</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>ReadData()</para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected virtual long ReadData()
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "ReadData", "<<Start>>");
			McTimeSeriesSingleCellTranInfo csCellTrn = null;

			HySCellData csDtRcd = null;

			// 入力された伝送情報を読み出す
			if ((csCellTrn = m_csInputData[0] as McTimeSeriesSingleCellTranInfo) != null)
			{
				OutputBasicRecord csRcd = null;  // 1レコード分のデータ
				long lDtNum = csCellTrn.SetCursorPlus(mInf.m_csLastINTime);
				// 最終時刻以後に情報が追加されているか？

				// 追加された情報分処理を行う
				for (long lDtLp = 0; lDtLp < lDtNum; lDtLp++)
				{  
					csDtRcd = csCellTrn.GetCursorData();
                    // m-suto 2015/12/25 start
                    // セル内情報数分の配列を確保するため、
                    // mInf.m_lAllCellNum ⇒ csDtRcd.m_lDim に変更(ループ回数も)
                    csRcd = new OutputBasicRecord(csDtRcd.m_lDim);  // 1レコード情報格納クラスの生成
                    for (int i = 0; i < csDtRcd.m_lDim; i++)
                    // m-suto 2015/12/25 end
                    {
						csRcd.m_dDT[i] = csDtRcd.m_dData[i];
					}
					
					csRcd.m_csTime = csCellTrn.GetCursorTime();

					// 読み出した情報を一時保管しておく
					mInf.m_csDataList.AddLast(csRcd);
					mInf.m_csLastINTime.SetTime(csRcd.m_csTime);

					csCellTrn.MoveCursorNext();
				}
			}
			else
			{
				McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "ReadData", "No InputCellData");
				return -1;
			}

			return 0;
		}

		/// <summary><para>method outline:</para>
		/// <para>プロパティ情報を設定する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>SetProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">セル型プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool SetProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			McModelPropertyInfo csPropInf = csCellMdlPropertyInfo as McModelPropertyInfo;
			if (csPropInf != null)
			{ 
				// 出力ファイルの設定
				csPropInf.GetInfo("m_csOutFileNameWithPath", ref this.mInf.m_csOutFileNameWithPath);

				csPropInf.GetInfo("m_lOutDataIdx", ref this.mInf.m_lOutDataIdx);
				csPropInf.GetInfo("m_lOriginYear", ref this.mInf.m_lOriginYear);
				csPropInf.GetInfo("m_lOriginMonth", ref this.mInf.m_lOriginMonth);
				csPropInf.GetInfo("m_lOriginDay", ref this.mInf.m_lOriginDay);

				long lDt = 0;
				// データ出力の時間間隔を取得
				csPropInf.GetInfo("m_csOutTimeRange", ref lDt);
				mInf.m_csOutTimeRange = new HySTime(lDt);

			}
			else
			{
				// エラーを返す
				McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "Cannot Read Property Info.");
				return false;
			}
			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>演算実行中断中のプロパティ等情報設定</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SetOnlineProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>演算中断中にプロパティ、初期値等を変更して動作させ場合等に使用する</para>
		/// </remarks>
		public override bool SetOnlineProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			return true;
		}
	}
}
