﻿using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.CoreImpl;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;


using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.CoreImpl.Data;
using CommonMP.HYMCO.CoreImpl.Tool;



namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>テスト用流量発生クラス</para>
	/// </summary>
	/// <remarks><para>history:</para>
	/// <para>[CommonMP][ver 1.0.0][2008/10/01][新規作成]</para>
	/// </remarks>
	public class JointPoint : McForecastModelBase
	{
		/// <summary>流域計算情報</summary>
		JointPointInfo mInf = null;
		
		// =========================
		// 演算実行前処理関連メソッド
		// =========================

		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>Initialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool Initialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "<<Init>>");

			mInf.m_dLastDataTime = m_csSimTime.TotalSec();
			mInf.m_dStepOutput = 0.0;
			mInf.m_dStepTotal = 0.0;
			HySCellData csCell = null;

			mInf.m_lDataKind = 0;

			// 入力データからデータ判別を行う
			for (long lP = 0; lP < lInputDataNum; lP++)
			{
				csCell = csInputCellData[lP].GetInterpolatedCell(0);
				if (csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎秒データ")) == true)
				{
					// 特に何もしない
				}
				else if (csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎分データ")) == true)
				{
					if (mInf.m_lDataKind < 1) mInf.m_lDataKind = 1;
				}
				else if (csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎時データ")) == true)
				{
					if (mInf.m_lDataKind < 2) mInf.m_lDataKind = 2;
				}
				else if (csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎日データ")) == true)
				{
					if (mInf.m_lDataKind < 3) mInf.m_lDataKind = 3;
				}
			}


			// データ判別結果から変換係数を算出する
			mInf.m_dK = new double[lInputDataNum];

			for (long lP = 0; lP < lInputDataNum; lP++)
			{
				mInf.m_dK[lP] = 1.0;
				if (csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎秒データ")) == true)
				{
					if (mInf.m_lDataKind > 0) mInf.m_dK[lP] *= 60.0;
					if (mInf.m_lDataKind > 1) mInf.m_dK[lP] *= 60.0;
					if (mInf.m_lDataKind > 2) mInf.m_dK[lP] *= 24.0;

				}
				else if (csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎分データ")) == true)
				{
					if (mInf.m_lDataKind > 1) mInf.m_dK[lP] *= 60.0;
					if (mInf.m_lDataKind > 2) mInf.m_dK[lP] *= 24.0;
				}
				else if (csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎時データ")) == true)
				{
					if (mInf.m_lDataKind > 2) mInf.m_dK[lP] *= 24.0;
				}
				else if (csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎日データ")) == true)
				{
					// 何もしない
				}
			}


			//Calculate(lInputDataNum, ref csInputCellData);

			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>OnlineInitialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>bool</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool OnlineInitialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			// Do Nothing
			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>入力側の接続情報チェック</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = ReceiveConnectionCheck(ref csErrorInf)</para>
		/// </example>
		/// <param name="csErrorInf">エラー出力</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>=true:正常、=false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>受信するデータが自モデルが期待している情報か否かをチェックする</para>
		/// </remarks>
		protected override bool ReceiveConnectionCheck(ref McStructErrorInfo csErrorInf, long lInputDataNum, McReceiveCellDataIF[] csInputCellData)
		{
			bool bRtn = true;

			if (lInputDataNum > 0)
			{

				for (long lP = 0; lP < lInputDataNum; lP++)
				{
					if((csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎秒データ")))
					 ||(csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎分データ")))
				 	 ||(csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎時データ")))
					 ||(csInputCellData[lP].GetReceivePatternID().Equals(new HySID("毎日データ"))))
					{
						// 正常
					}
					else
					{
						// エラー
						csErrorInf.AddCheckErrorData(this.GetID(), ToolDefine.JOINT_POINT_KIND,
								"Unexpected receive data type. (Received from " + csInputCellData[lP].GetUpperElementID().ToString() + ")");
						bRtn = false;
					}
				}
			}
			return bRtn;
		}
		/// <summary><para>method outline:</para>
		/// <para>出力側の接続情報チェック</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SendConnectionCheck(ref csErrorInf)</para>
		/// </example>
		/// <param name="csErrorInf">エラー出力</param>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=true:正常、=false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>送信端子に設定されている伝送データが自モデルが期待している情報か否かをチェックする</para>
		/// </remarks>
		protected override bool SendConnectionCheck(ref McStructErrorInfo csErrorInf, long lOutputDataNum, McSendCellDataIF[] csOutputCellData)
		{
			bool bRtn = true;

			if (lOutputDataNum == 0)
			{
				// 警告（エラーではない）
				csErrorInf.AddCheckWarningData(this.GetID(), ToolDefine.COMPARE_DATA_KIND, "No send port.");
			}
			for (long lP = 0; lP < lOutputDataNum; lP++)
			{
				if (csOutputCellData[lP].GetSendPatternID().Equals(ToolDefine.IN_OUT_PATTERN_ANY))
				{
				}
				else
				{
					// エラー
					csErrorInf.AddCheckErrorData(this.GetID(), ToolDefine.COMPARE_DATA_KIND,
											"Unexpected send data type. (Send To " + csOutputCellData[lP].GetLowerElementID().ToString() + ")");
					bRtn = false;
				}
			}
			return bRtn;
		}

		// =======================
		// 演算実行処理関連メソッド
		// =======================

		/// <summary><para>method outline:</para>
		/// <para>モデル演算</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>long lRtn = Calculate(ref csInputCellData)</para>
		/// </example>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override long Calculate(long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Calculate", "<<Start>>");

			HySCellData csCell = null;
			double dt = m_csDltTime.TotalSec();

			if (m_csSimTime.TotalSec() + dt >= mInf.m_dLastDataTime + mInf.m_dStepTime)
			{
				// 時間を超えたのでデータフュージョンフラグを立てる
				
				// ステップごとに動作するため、目標時間を超える可能性がある
				// オーバーした分は次のステップに回す
				double dOver = (mInf.m_dLastDataTime + mInf.m_dStepTime - m_csSimTime.TotalSec() - dt) / mInf.m_dStepTime;
				
				if (dOver > 1.0)
				{
					// ステップに占める比率が1.0を超えるのは異常
					return -1;
				}

				// アウトプットするデータを用意する
				mInf.m_dStepOutput = mInf.m_dStepTotal;
				mInf.m_dStepTotal = 0.0;

				for (long lP = 0; lP < lInputDataNum; lP++)
				{
					csCell = csInputCellData[lP].GetInterpolatedCell(0);
					mInf.m_dStepOutput += (1 - dOver) * csInputCellData[lP].Data(csCell, 0) * mInf.m_dK[lP];
					mInf.m_dStepTotal += dOver * csInputCellData[lP].Data(csCell, 0) * mInf.m_dK[lP];
				}
				mInf.m_dLastDataTime += mInf.m_dStepTime;
			
			}
			else
			{
				for (long lP = 0; lP < lInputDataNum; lP++)
				{
					csCell = csInputCellData[lP].GetInterpolatedCell(0);
					mInf.m_dStepTotal += csInputCellData[lP].Data(csCell, 0) * mInf.m_dK[lP];// 秒
				}
			}

			return 0;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデル演算結果を外部のエレメントに対して公開する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>long lRtn = DataFusion( lOutputDataNum, ref csOutputCellData)</para>
		/// </example>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override long DataFusion(long lOutputDataNum, ref McSendCellDataIF[] csOutputCellData)
		{
			HySCellData[] csSndCellData = null;

			// 期間出力時間がきたので期間出力を行う
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "DataFusion", "<<Start>>");

			for (long lP = 0; lP < lOutputDataNum; lP++)
			{
				//csOutputCellData[lP].SetCurrentTime(new HySTime(mInf.m_dLastDataTime + mInf.m_dStepTime));
				csSndCellData = csOutputCellData[lP].PrepareSendCellD1();
				csSndCellData[0].m_dData[0] = mInf.m_dStepOutput / mInf.m_dStepTime * this.m_csDltTime.TotalSec();
				csSndCellData[0].m_dData[1] = mInf.m_dStepOutput * this.m_csDltTime.TotalSec();
				csSndCellData[0].m_dData[2] = mInf.m_dStepTime;
				//csOutputCellData[lP].SendData();
			}

			return 0;
		}
		// ====================
		// その他必要なメソッド
		// ====================

		/// <summary><para>method outline:</para>
		/// <para>プロパティ情報を設定する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>SetProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">セル型プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool SetProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "<<Init>>");
			// 使用しやすいようにキャストしておく
			mInf = (JointPointInfo)m_csCalInfo;

			double dRef = 0.0;

			McCellModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McCellModelPropertyInfo;
			if (csPrptyInfo != null)
			{
				this.m_csDltTime = new HySTime(csPrptyInfo.GetStepTime());
				// 計算パラメーター
				csPrptyInfo.GetInfo("m_dStepTime", ref dRef);
				mInf.m_dStepTime = dRef;
			}

			/*
			if (dRef > 0 && dRef < this.m_csOrgDltTime.TotalSec())
			{
				SetDeltaTime(new HySTime(dRef));
			}
			else
			{
				// ステップ時刻の設定なし
				SetDeltaTime(this.m_csOrgDltTime);
			}
			*/
			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>演算実行中断中のプロパティ等情報設定</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SetOnlineProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>演算中断中にプロパティ、初期値等を変更して動作させ場合等に使用する</para>
		/// </remarks>
		public override bool SetOnlineProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			// プロパティ設定
			McCellModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McCellModelPropertyInfo;
			if (csPrptyInfo != null)
			{
				// To Do
				// モデルの係数等、必要な情報に対して、引数で与えられたプロパティ情報の内容を読み取って、設定していきます。
				// 但し、実行中に変更して困る情報は設定しないで下さい。
			}
			return true;
		}
		// 
	}
}
